#!/usr/bin/env python
# -*- coding: utf-8 -*-

from time import time
import logging

_logger = logging.getLogger(__name__)
ENABLE_DEBUG_LOG = True

ERROR_DETECTION_TIMEOUT = 4     # nach wepotec LH V14 Kap. 8.6.8


class FanControl(object):
    """!
    Die Lüftersteuerung ist für die Ausgabe der korrekten Signale an die Lüfter zuständig.
    Sie überwacht, ob sich ein aktiver Lüfter tatsächlich dreht.
    """

    def __init__(self, daio, analog_input):
        """!
        Initialisiert die Lüftersteuerung.
        @param daio: Instanz der DIO
        @param analog_input: Instanz der AIO
        """
        _logger.debug("FanControl init")
        self._name = "FanControl"

        self._daio = daio
        self._analog_input = analog_input

        self._fan1_error_active = False
        self._fan2_error_active = False
        self._fan3_error_active = False

        self._fan1_active = False
        self._fan2_active = False
        self._fan3_active = False

        self._fan1_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
        self._fan2_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
        self._fan3_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT

    def setFan1(self, value):
        """!
        Setzt den Wert für Lüfter 1.
        @param value: Wert, wie schnell sich Lüfter 1 drehen soll [0..100%]
        """
        if value == 0:
            self._fan1_active = False
            self._daio.fan1_on(False)
            self._daio.fan1_set(0)
        else:
            if not self._fan1_active:
                # reset the check error here
                self._fan1_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            self._fan1_active = True
            self._daio.fan1_on(True)
            self._daio.fan1_set(value)

    def setFan2(self, value):
        """!
        Setzt den Wert für Lüfter 2.
        @param value: Wert, wie schnell sich Lüfter 2 drehen soll [0..100%]
        """
        if value == 0:
            self._fan2_active = False
            self._daio.fan2_on(False)
            self._daio.fan2_set(0)
        else:
            if not self._fan2_active:
                # reset the check error here
                self._fan2_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            self._fan2_active = True
            self._daio.fan2_on(True)
            self._daio.fan2_set(value)

    def setFan3(self, value):
        """!
        Setzt den Wert für Lüfter 3.
        @param value: Wert, wie schnell sich Lüfter 3 drehen soll [0..100%]
        """
        if value == 0:
            self._fan3_active = False
            self._daio.fan3_on(False)
            self._daio.fan3_set(0)
        else:
            if not self._fan3_active:
                # reset the check error here
                self._fan3_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            self._fan3_active = True
            self._daio.fan3_on(True)
            self._daio.fan3_set(value)

    @property
    def fan1_error(self):
        """!
        Zustand Lüfter 1.
        @return True: Fehler bei Lüfter 1
        """
        return self._fan1_error_active

    @property
    def fan2_error(self):
        """!
        Zustand Lüfter 2.
        @return True: Fehler bei Lüfter 2
        """
        return self._fan2_error_active

    @property
    def fan3_error(self):
        """!
        Zustand Lüfter 3.
        @return True: Fehler bei Lüfter 3
        """
        return self._fan3_error_active

    def __call__(self):
        """!
        Zyklische Lüfterüberwachung.
        Wenn einer der Lüfter stehen bleibt, wird das jeweilige Fehlerflag gesetzt.
        Funktion muss zyklisch aufgerufen werden.
        """
        if self._fan1_active:
            if self._daio.is_fan1_running():
                self._fan1_error_active = False
                self._fan1_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            elif not self._fan1_error_active and self._fan1_check_error_timeout < time():
                if ENABLE_DEBUG_LOG:
                    _logger.debug("fan 1 error")
                self._fan1_error_active = True

        if self._fan2_active:
            if self._daio.is_fan2_running():
                self._fan2_error_active = False
                self._fan2_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            elif not self._fan2_error_active and self._fan2_check_error_timeout < time():
                if ENABLE_DEBUG_LOG:
                    _logger.debug("fan 2 error")
                self._fan2_error_active = True

        if self._fan3_active:
            if self._daio.is_fan3_running():
                self._fan3_error_active = False
                self._fan3_check_error_timeout = time() + ERROR_DETECTION_TIMEOUT
            elif not self._fan3_error_active and self._fan3_check_error_timeout < time():
                if ENABLE_DEBUG_LOG:
                    _logger.debug("fan 3 error")
                self._fan3_error_active = True
